@@ -380,8 +380,8 @@ floating_widget_change_params (FloatingWidget *floating,
 			       int sz,
 			       gboolean hidebuttons_enabled,
 			       gboolean hidebutton_pixmap_enabled,
-			       PanelBackType back_type,
-			       char *back_pixmap,
+			       PanelBackgroundType back_type,
+			       const char *back_pixmap,
 			       gboolean fit_pixmap_bg,
 			       gboolean stretch_pixmap_bg,
 			       gboolean rotate_pixmap_bg,
@@ -441,17 +441,17 @@ floating_widget_change_coords (FloatingWidget *floating,
 					       panel->sz,
 					       basep->hidebuttons_enabled,
 					       basep->hidebutton_pixmaps_enabled,
-					       panel->back_type,
-					       panel->back_pixmap,
-					       panel->fit_pixmap_bg,
-					       panel->stretch_pixmap_bg,
-					       panel->rotate_pixmap_bg,
-					       &panel->back_color);
+					       panel->background.type,
+					       panel->background.image,
+					       panel->background.fit_image,
+					       panel->background.stretch_image,
+					       panel->background.rotate_image,
+					       &panel->background.color);
 	}
 }
 
 GtkWidget *
-floating_widget_new (gchar *panel_id,
+floating_widget_new (const char *panel_id,
 		     int screen,
 		     int monitor,
 		     gint16 x,
@@ -462,8 +462,8 @@ floating_widget_new (gchar *panel_id,
 		     int sz,
 		     gboolean hidebuttons_enabled,
 		     gboolean hidebutton_pixmap_enabled,
-		     PanelBackType back_type,
-		     char *back_pixmap,
+		     PanelBackgroundType back_type,
+		     const char *back_pixmap,
 		     gboolean fit_pixmap_bg,
 		     gboolean stretch_pixmap_bg,
 		     gboolean rotate_pixmap_bg,
